@@ -1,6 +1,6 @@
 /*
 
-   $Id: dbdimp.c,v 1.110 2005/04/01 23:37:23 turnstep Exp $
+   $Id: dbdimp.c,v 1.115 2005/04/05 01:40:42 turnstep Exp $
 
    Copyright (c) 2002-2005 PostgreSQL Global Development Group
    Portions Copyright (c) 2002 Jeffrey W. Baker
@@ -50,6 +50,8 @@ PGresult *PQexecParams() { croak("Called wrong PQexecParams\n"); }
 Oid PQftable() { return InvalidOid; }
 int PQftablecol() { return 0; }
 int PQsetErrorVerbosity() { return 0; }
+int PQprotocolVersion() { return 0; }
+PGTransactionStatusType PQtransactionStatus() { return PQTRANS_UNKNOWN; }
 #define PG_DIAG_SQLSTATE 'C'
 #endif
 
@@ -147,7 +149,6 @@ void pg_error (h, error_num, error_msg)
 	sv_setiv(DBIc_ERR(imp_xxh), (IV)error_num);		 /* set err early */
 	sv_setpv(DBIc_ERRSTR(imp_xxh), (char*)err);
 	sv_setpvn(DBIc_STATE(imp_xxh), (char*)imp_dbh->sqlstate, 5);
-	DBIh_EVENT2(h, ERROR_event, DBIc_ERR(imp_xxh), DBIc_ERRSTR(imp_xxh));
 	if (dbis->debug >= 3) {
 		PerlIO_printf(DBILOGFP, "%s error %d recorded: %s\n",
 									err, error_num, SvPV_nolen(DBIc_ERRSTR(imp_xxh)));
@@ -377,7 +378,9 @@ int dbd_db_rollback_commit (dbh, imp_dbh, action)
 	/* We only perform these actions if we need to. For newer servers, we 
 		 ask it for the status directly and double-check things */
 
-#if PGLIBVERSION >= 70400
+#if PGLIBVERSION < 70400
+	tstatus = 0; /* Make compiler happy */
+#else
 	tstatus = dbd_db_txn_status(imp_dbh);
 	if (PQTRANS_IDLE == tstatus) { /* Not in a transaction */
 		if (imp_dbh->done_begin) {
@@ -546,9 +549,7 @@ int dbd_db_STORE_attrib (dbh, imp_dbh, keysv, valuesv)
 			imp_dbh->prepare_now = newval ? 1 : 0;
 		}
 	}
-	else {
-		return 0;
-	}
+	return 0;
 
 } /* end of dbd_db_STORE_attrib */
 
@@ -639,8 +640,6 @@ int dbd_discon_all (drh, imp_drh)
 		sv_setiv(DBIc_ERR(imp_drh), (IV)1);
 		sv_setpv(DBIc_ERRSTR(imp_drh),
 						 (char*)"disconnect_all not implemented");
-		DBIh_EVENT2(drh, ERROR_event,
-								DBIc_ERR(imp_drh), DBIc_ERRSTR(imp_drh));
 	}
 	return DBDPG_FALSE;
 
@@ -1259,7 +1258,7 @@ int dbd_st_prepare_statement (sth, imp_sth)
 	else {
 		if (imp_sth->numbound) {
 			params = imp_sth->numphs;
-			paramTypes = calloc(imp_sth->numphs, sizeof(*paramTypes));
+			Newz(0, paramTypes, imp_sth->numphs, Oid);
 			for (x=0,currph=imp_sth->ph; NULL != currph; currph=currph->nextph) {
 				paramTypes[x++] = currph->defaultval ? 0 : currph->bind_type->type_id;
 			}
@@ -1304,7 +1303,7 @@ int dbd_bind_ph (sth, imp_sth, ph_name, newvalue, sql_type, attribs, is_inout, m
 	SV **svp;
 	bool reprepare = 0;
 	int pg_type = 0;
-	char *value_string;
+	char *value_string = NULL;
 
 	if (dbis->debug >= 4) {
 		PerlIO_printf(DBILOGFP, "dbd_bind_ph\n");
@@ -1482,7 +1481,7 @@ int dbd_st_execute (sth, imp_sth) /* <= -2:error, >=0:ok row count, (-1=unknown
 	ph_t *currph;
 	ExecStatusType status = -1;
 	STRLEN execsize, x;
-	const char **paramValues;
+	const char **paramValues = NULL;
 	int *paramLengths = NULL, *paramFormats = NULL;
 	Oid *paramTypes = NULL;
 	seg_t *currseg;
@@ -1556,7 +1555,7 @@ int dbd_st_execute (sth, imp_sth) /* <= -2:error, >=0:ok row count, (-1=unknown
 	}
 	else { /* We are using a server that can handle PQexecParams/PQexecPrepared */
 		/* Put all values into an array to pass to PQexecPrepared */
-		paramValues = calloc(imp_sth->numphs, sizeof(*paramValues));
+		Newz(0, paramValues, imp_sth->numphs, const char *); /* freed below */
 		for (x=0,currph=imp_sth->ph; NULL != currph; currph=currph->nextph) {
 			paramValues[x++] = currph->value;
 		}
@@ -1564,8 +1563,8 @@ int dbd_st_execute (sth, imp_sth) /* <= -2:error, >=0:ok row count, (-1=unknown
 		/* Binary or regular? */
 
 		if (imp_sth->has_binary) {
-			paramLengths = calloc(imp_sth->numphs, sizeof(*paramLengths));
-			paramFormats = calloc(imp_sth->numphs, sizeof(*paramFormats));
+			Newz(0, paramLengths, imp_sth->numphs, int); /* freed below */
+			Newz(0, paramFormats, imp_sth->numphs, int); /* freed below */
 			for (x=0,currph=imp_sth->ph; NULL != currph; currph=currph->nextph,x++) {
 				if (BYTEAOID==currph->bind_type->type_id) {
 					paramLengths[x] = currph->valuelen;
@@ -1680,7 +1679,7 @@ int dbd_st_execute (sth, imp_sth) /* <= -2:error, >=0:ok row count, (-1=unknown
 			statement[execsize] = '\0';
 			
 			/* Populate paramTypes */
-			paramTypes = calloc(imp_sth->numphs, sizeof(*paramTypes));
+			Newz(0, paramTypes, imp_sth->numphs, Oid);
 			for (x=0,currph=imp_sth->ph; NULL != currph; currph=currph->nextph) {
 				paramTypes[x++] = currph->defaultval ? 0 : currph->bind_type->type_id;
 			}
@@ -1803,7 +1802,7 @@ int dbd_st_execute (sth, imp_sth) /* <= -2:error, >=0:ok row count, (-1=unknown
 
 
 /* ================================================================== */
-is_high_bit_set(val)
+int is_high_bit_set(val)
 		 char *val;
 {
 	while (*val++)
@@ -1821,7 +1820,7 @@ AV * dbd_st_fetch (sth, imp_sth)
 	int num_fields;
 	char *value;
 	char *p;
-	int i, pg_type, chopblanks;
+	int i, chopblanks;
 	STRLEN value_len;
 	STRLEN len;
 	AV *av;
@@ -1850,7 +1849,7 @@ AV * dbd_st_fetch (sth, imp_sth)
 
 	/* Set up the type_info array if we have not seen it yet */
 	if (NULL==imp_sth->type_info) {
-		imp_sth->type_info = calloc(num_fields, sizeof(*imp_sth->type_info)); /* freed in dbd_st_destroy */
+		Newz(0, imp_sth->type_info, num_fields, sql_type_info_t*); /* freed in dbd_st_destroy */
 		for (i = 0; i < num_fields; ++i) {
 			imp_sth->type_info[i] = pg_type_data(PQftype(imp_sth->result, i));
 		}
@@ -1900,7 +1899,7 @@ AV * dbd_st_fetch (sth, imp_sth)
 				case TEXTOID:
 				case BPCHAROID:
 				case VARCHAROID:
-					if (is_high_bit_set(value) && is_utf8_string(value, value_len)) {
+					if (is_high_bit_set(value) && is_utf8_string((unsigned char*)value, value_len)) {
 						SvUTF8_on(sv);
 					}
 				}
@@ -1954,8 +1953,8 @@ int dbd_st_deallocate_statement (sth, imp_sth)
 		 SV *sth;
 		 imp_sth_t *imp_sth;
 {
+	char tempsqlstate[6];
 	char *stmt;
-	PGresult *result;
 	ExecStatusType status;
 	PGTransactionStatusType tstatus;
 	D_imp_dbh_from_sth;
@@ -1965,6 +1964,8 @@ int dbd_st_deallocate_statement (sth, imp_sth)
 	if (NULL == imp_dbh->conn || NULL == imp_sth->prepare_name)
 		return 0;
 	
+	tempsqlstate[0] = '\0';
+
 	/* What is our status? */
 	tstatus = dbd_db_txn_status(imp_dbh);
 	if (dbis->debug >= 5)
@@ -1973,18 +1974,19 @@ int dbd_st_deallocate_statement (sth, imp_sth)
 	/* If we are in a failed transaction, rollback before deallocating */
 	if (PQTRANS_INERROR == tstatus) {
 		if (dbis->debug >= 4)
-			PerlIO_printf(DBILOGFP, "  dbdpg: Issuing rollback before deallocate\n", tstatus);
+			PerlIO_printf(DBILOGFP, "  dbdpg: Issuing rollback before deallocate\n");
 		{
 			/* If a savepoint has been set, rollback to the last savepoint instead of the entire transaction */
 			I32	alen = av_len(imp_dbh->savepoints);
 			if (alen > -1) {
-				SV		*sp;
+				SV		*sp = Nullsv;
 				char	*cmd;
 				New(0, cmd, SvLEN(sp) + 13, char); /* Freed below */
+				sp = *av_fetch(imp_dbh->savepoints, alen, 0);
 				if (dbis->debug >= 4)
-					PerlIO_printf(DBILOGFP, "  dbdpg: Issuing rollback before deallocate\n", tstatus);
-				sp = av_pop(imp_dbh->savepoints);
+					PerlIO_printf(DBILOGFP, "  dbdpg: Rolling back to savepoint %s\n", SvPV_nolen(sp));
 				sprintf(cmd,"rollback to %s",SvPV_nolen(sp));
+				strcpy(tempsqlstate, imp_dbh->sqlstate);
 				status = _result(imp_dbh, cmd);
 				Safefree(cmd);
 			}
@@ -2018,6 +2020,8 @@ int dbd_st_deallocate_statement (sth, imp_sth)
 	}
 
 	imp_sth->prepare_name = NULL;
+	if (tempsqlstate[0])
+		strcpy(imp_dbh->sqlstate, tempsqlstate);
 
 	return 0;
 
@@ -2109,9 +2113,7 @@ int dbd_st_STORE_attrib (sth, imp_sth, keysv, valuesv)
 		Copy(value, imp_sth->prepare_name, vl, char);
 		imp_sth->prepare_name[vl] = '\0';
 	}
-	else {
-		return 0;
-	}
+	return 0;
 
 } /* end of sbs_st_STORE_attrib */
 
@@ -2294,7 +2296,7 @@ SV * dbd_st_FETCH_attrib (sth, imp_sth, keysv)
 int
 pg_db_putline (dbh, buffer)
 		SV *dbh;
-		char *buffer;
+		const char *buffer;
 {
 		D_imp_dbh(dbh);
 		int result;
@@ -2306,6 +2308,7 @@ pg_db_putline (dbh, buffer)
 #if PGLIBVERSION < 70400
 		if (dbis->debug >= 4)
 			PerlIO_printf(DBILOGFP, "  dbdpg: PQputline\n");
+		result = 0; /* Make compilers happy */
 		return PQputline(imp_dbh->conn, buffer);
 #else
 		if (dbis->debug >= 4)
@@ -2349,6 +2352,7 @@ pg_db_getline (dbh, buffer, length)
 			PQendcopy(imp_dbh->conn);
 			return -1;
 		}
+		tempbuf = NULL; /* Make compilers happy */
 		return result;
 #else
 		result = PQgetCopyData(imp_dbh->conn, &tempbuf, 0);
@@ -2378,7 +2382,6 @@ pg_db_endcopy (dbh)
 		D_imp_dbh(dbh);
 		int res;
 		PGresult *result;
-		ExecStatusType status;
 
 		if (0==imp_dbh->copystate)
 			croak("pg_endcopy cannot be called until a COPY is issued");
@@ -2388,6 +2391,7 @@ pg_db_endcopy (dbh)
 #if PGLIBVERSION < 70400
 		if (PGRES_COPY_IN == imp_dbh->copystate)
 			PQputline(imp_dbh->conn, "\\.\n");
+		result = 0; /* Make compiler happy */
 		res = PQendcopy(imp_dbh->conn);
 #else
 		if (PGRES_COPY_IN == imp_dbh->copystate) {
@@ -2447,7 +2451,6 @@ pg_db_savepoint (dbh, imp_dbh, savepoint)
 		 imp_dbh_t *imp_dbh;
 		 char * savepoint;
 {
-	PGTransactionStatusType tstatus;
 	ExecStatusType status;
 	char *action;
 
@@ -2496,7 +2499,6 @@ int pg_db_rollback_to (dbh, imp_dbh, savepoint)
 		 imp_dbh_t *imp_dbh;
 		 char * savepoint;
 {
-	PGTransactionStatusType tstatus;
 	ExecStatusType status;
 	I32 i;
 	char *action;
@@ -2541,7 +2543,6 @@ int pg_db_release (dbh, imp_dbh, savepoint)
 		 imp_dbh_t *imp_dbh;
 		 char * savepoint;
 {
-	PGTransactionStatusType tstatus;
 	ExecStatusType status;
 	I32 i;
 	char *action;
